<?php

namespace FiloBlu\Refilo\Model\Exporter;

use FiloBlu\Refilo\Model\Adapter\Mongo;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\App\State;
use Magento\Store\Api\StoreConfigManagerInterface;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class Translations
 * @package FiloBlu\Refilo\Model\Exporter
 */
class Search extends AbstractExporter
{
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * Menu constructor.
     * @param ResourceConnection $resourceConnection
     * @param ScopeConfigInterface $scopeConfig
     * @param Mongo $mongo
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(
        ResourceConnection $resourceConnection,
        ScopeConfigInterface $scopeConfig,
        Mongo $mongo,
        StoreManagerInterface $storeManager
    ) {
        parent::__construct($mongo, $scopeConfig);
        $this->resourceConnection = $resourceConnection;
        $this->storeManager = $storeManager;
    }

    /**
     * @return iterable
     */
    public function getCollections(): iterable
    {
        yield 'search';
    }

    public function preExecute()
    {
        $this->getConnection()->dropCollection(
            'search',
            $this->getDatabase(),
        );
    }

    public function execute()
    {

        $allStores = $this->storeManager->getStores();

        $bulkData = [];
        $_id = 1;
        $query = "SELECT * FROM search_query";

        $result = $this->resourceConnection->getConnection()->fetchAll($query);
        foreach ($result as $item) {
            $item = array_map('json_decode', $item);
            $item["_id"] = $_id;
            $item["store"] = $allStores[(int)$item["store_id"]]->getData('code');
            $bulkData[] = $item;
            $_id++;
        }

        try {
            $this->getConnection()->update(
                '_id',
                $bulkData,
                'search',
                true,
                $this->getDatabase()
            );
        } catch (\MongoDB\Driver\Exception\Exception $e) {
            echo $e->getMessage();
        }
    }
}
